/*
 * Copyright (C) 2016 Simon Fels <morphis@gravedo.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef ANBOX_ANDROID_INTENT_H_
#define ANBOX_ANDROID_INTENT_H_

#include <string>
#include <vector>

namespace anbox::android {
struct Intent {
  std::string action;
  std::string uri;
  std::string type;
  int flags = 0;
  std::string package;
  std::string component;
  std::vector<std::string> categories;

  bool valid() const;
};

std::ostream &operator<<(std::ostream &out, const Intent &intent);
}
#endif
