// Generated Code - DO NOT EDIT !!
// generated by 'emugen'
#ifndef __GUARD_gl2_opcodes_h_
#define __GUARD_gl2_opcodes_h_

#define OP_glActiveTexture 					2048
#define OP_glAttachShader 					2049
#define OP_glBindAttribLocation 					2050
#define OP_glBindBuffer 					2051
#define OP_glBindFramebuffer 					2052
#define OP_glBindRenderbuffer 					2053
#define OP_glBindTexture 					2054
#define OP_glBlendColor 					2055
#define OP_glBlendEquation 					2056
#define OP_glBlendEquationSeparate 					2057
#define OP_glBlendFunc 					2058
#define OP_glBlendFuncSeparate 					2059
#define OP_glBufferData 					2060
#define OP_glBufferSubData 					2061
#define OP_glCheckFramebufferStatus 					2062
#define OP_glClear 					2063
#define OP_glClearColor 					2064
#define OP_glClearDepthf 					2065
#define OP_glClearStencil 					2066
#define OP_glColorMask 					2067
#define OP_glCompileShader 					2068
#define OP_glCompressedTexImage2D 					2069
#define OP_glCompressedTexSubImage2D 					2070
#define OP_glCopyTexImage2D 					2071
#define OP_glCopyTexSubImage2D 					2072
#define OP_glCreateProgram 					2073
#define OP_glCreateShader 					2074
#define OP_glCullFace 					2075
#define OP_glDeleteBuffers 					2076
#define OP_glDeleteFramebuffers 					2077
#define OP_glDeleteProgram 					2078
#define OP_glDeleteRenderbuffers 					2079
#define OP_glDeleteShader 					2080
#define OP_glDeleteTextures 					2081
#define OP_glDepthFunc 					2082
#define OP_glDepthMask 					2083
#define OP_glDepthRangef 					2084
#define OP_glDetachShader 					2085
#define OP_glDisable 					2086
#define OP_glDisableVertexAttribArray 					2087
#define OP_glDrawArrays 					2088
#define OP_glDrawElements 					2089
#define OP_glEnable 					2090
#define OP_glEnableVertexAttribArray 					2091
#define OP_glFinish 					2092
#define OP_glFlush 					2093
#define OP_glFramebufferRenderbuffer 					2094
#define OP_glFramebufferTexture2D 					2095
#define OP_glFrontFace 					2096
#define OP_glGenBuffers 					2097
#define OP_glGenerateMipmap 					2098
#define OP_glGenFramebuffers 					2099
#define OP_glGenRenderbuffers 					2100
#define OP_glGenTextures 					2101
#define OP_glGetActiveAttrib 					2102
#define OP_glGetActiveUniform 					2103
#define OP_glGetAttachedShaders 					2104
#define OP_glGetAttribLocation 					2105
#define OP_glGetBooleanv 					2106
#define OP_glGetBufferParameteriv 					2107
#define OP_glGetError 					2108
#define OP_glGetFloatv 					2109
#define OP_glGetFramebufferAttachmentParameteriv 					2110
#define OP_glGetIntegerv 					2111
#define OP_glGetProgramiv 					2112
#define OP_glGetProgramInfoLog 					2113
#define OP_glGetRenderbufferParameteriv 					2114
#define OP_glGetShaderiv 					2115
#define OP_glGetShaderInfoLog 					2116
#define OP_glGetShaderPrecisionFormat 					2117
#define OP_glGetShaderSource 					2118
#define OP_glGetString 					2119
#define OP_glGetTexParameterfv 					2120
#define OP_glGetTexParameteriv 					2121
#define OP_glGetUniformfv 					2122
#define OP_glGetUniformiv 					2123
#define OP_glGetUniformLocation 					2124
#define OP_glGetVertexAttribfv 					2125
#define OP_glGetVertexAttribiv 					2126
#define OP_glGetVertexAttribPointerv 					2127
#define OP_glHint 					2128
#define OP_glIsBuffer 					2129
#define OP_glIsEnabled 					2130
#define OP_glIsFramebuffer 					2131
#define OP_glIsProgram 					2132
#define OP_glIsRenderbuffer 					2133
#define OP_glIsShader 					2134
#define OP_glIsTexture 					2135
#define OP_glLineWidth 					2136
#define OP_glLinkProgram 					2137
#define OP_glPixelStorei 					2138
#define OP_glPolygonOffset 					2139
#define OP_glReadPixels 					2140
#define OP_glReleaseShaderCompiler 					2141
#define OP_glRenderbufferStorage 					2142
#define OP_glSampleCoverage 					2143
#define OP_glScissor 					2144
#define OP_glShaderBinary 					2145
#define OP_glShaderSource 					2146
#define OP_glStencilFunc 					2147
#define OP_glStencilFuncSeparate 					2148
#define OP_glStencilMask 					2149
#define OP_glStencilMaskSeparate 					2150
#define OP_glStencilOp 					2151
#define OP_glStencilOpSeparate 					2152
#define OP_glTexImage2D 					2153
#define OP_glTexParameterf 					2154
#define OP_glTexParameterfv 					2155
#define OP_glTexParameteri 					2156
#define OP_glTexParameteriv 					2157
#define OP_glTexSubImage2D 					2158
#define OP_glUniform1f 					2159
#define OP_glUniform1fv 					2160
#define OP_glUniform1i 					2161
#define OP_glUniform1iv 					2162
#define OP_glUniform2f 					2163
#define OP_glUniform2fv 					2164
#define OP_glUniform2i 					2165
#define OP_glUniform2iv 					2166
#define OP_glUniform3f 					2167
#define OP_glUniform3fv 					2168
#define OP_glUniform3i 					2169
#define OP_glUniform3iv 					2170
#define OP_glUniform4f 					2171
#define OP_glUniform4fv 					2172
#define OP_glUniform4i 					2173
#define OP_glUniform4iv 					2174
#define OP_glUniformMatrix2fv 					2175
#define OP_glUniformMatrix3fv 					2176
#define OP_glUniformMatrix4fv 					2177
#define OP_glUseProgram 					2178
#define OP_glValidateProgram 					2179
#define OP_glVertexAttrib1f 					2180
#define OP_glVertexAttrib1fv 					2181
#define OP_glVertexAttrib2f 					2182
#define OP_glVertexAttrib2fv 					2183
#define OP_glVertexAttrib3f 					2184
#define OP_glVertexAttrib3fv 					2185
#define OP_glVertexAttrib4f 					2186
#define OP_glVertexAttrib4fv 					2187
#define OP_glVertexAttribPointer 					2188
#define OP_glViewport 					2189
#define OP_glEGLImageTargetTexture2DOES 					2190
#define OP_glEGLImageTargetRenderbufferStorageOES 					2191
#define OP_glGetProgramBinaryOES 					2192
#define OP_glProgramBinaryOES 					2193
#define OP_glMapBufferOES 					2194
#define OP_glUnmapBufferOES 					2195
#define OP_glTexImage3DOES 					2196
#define OP_glTexSubImage3DOES 					2197
#define OP_glCopyTexSubImage3DOES 					2198
#define OP_glCompressedTexImage3DOES 					2199
#define OP_glCompressedTexSubImage3DOES 					2200
#define OP_glFramebufferTexture3DOES 					2201
#define OP_glBindVertexArrayOES 					2202
#define OP_glDeleteVertexArraysOES 					2203
#define OP_glGenVertexArraysOES 					2204
#define OP_glIsVertexArrayOES 					2205
#define OP_glDiscardFramebufferEXT 					2206
#define OP_glMultiDrawArraysEXT 					2207
#define OP_glMultiDrawElementsEXT 					2208
#define OP_glGetPerfMonitorGroupsAMD 					2209
#define OP_glGetPerfMonitorCountersAMD 					2210
#define OP_glGetPerfMonitorGroupStringAMD 					2211
#define OP_glGetPerfMonitorCounterStringAMD 					2212
#define OP_glGetPerfMonitorCounterInfoAMD 					2213
#define OP_glGenPerfMonitorsAMD 					2214
#define OP_glDeletePerfMonitorsAMD 					2215
#define OP_glSelectPerfMonitorCountersAMD 					2216
#define OP_glBeginPerfMonitorAMD 					2217
#define OP_glEndPerfMonitorAMD 					2218
#define OP_glGetPerfMonitorCounterDataAMD 					2219
#define OP_glRenderbufferStorageMultisampleIMG 					2220
#define OP_glFramebufferTexture2DMultisampleIMG 					2221
#define OP_glDeleteFencesNV 					2222
#define OP_glGenFencesNV 					2223
#define OP_glIsFenceNV 					2224
#define OP_glTestFenceNV 					2225
#define OP_glGetFenceivNV 					2226
#define OP_glFinishFenceNV 					2227
#define OP_glSetFenceNV 					2228
#define OP_glCoverageMaskNV 					2229
#define OP_glCoverageOperationNV 					2230
#define OP_glGetDriverControlsQCOM 					2231
#define OP_glGetDriverControlStringQCOM 					2232
#define OP_glEnableDriverControlQCOM 					2233
#define OP_glDisableDriverControlQCOM 					2234
#define OP_glExtGetTexturesQCOM 					2235
#define OP_glExtGetBuffersQCOM 					2236
#define OP_glExtGetRenderbuffersQCOM 					2237
#define OP_glExtGetFramebuffersQCOM 					2238
#define OP_glExtGetTexLevelParameterivQCOM 					2239
#define OP_glExtTexObjectStateOverrideiQCOM 					2240
#define OP_glExtGetTexSubImageQCOM 					2241
#define OP_glExtGetBufferPointervQCOM 					2242
#define OP_glExtGetShadersQCOM 					2243
#define OP_glExtGetProgramsQCOM 					2244
#define OP_glExtIsProgramBinaryQCOM 					2245
#define OP_glExtGetProgramBinarySourceQCOM 					2246
#define OP_glStartTilingQCOM 					2247
#define OP_glEndTilingQCOM 					2248
#define OP_glVertexAttribPointerData 					2249
#define OP_glVertexAttribPointerOffset 					2250
#define OP_glDrawElementsOffset 					2251
#define OP_glDrawElementsData 					2252
#define OP_glGetCompressedTextureFormats 					2253
#define OP_glShaderString 					2254
#define OP_glFinishRoundTrip 					2255
#define OP_last 					2256


#endif
