// Generated Code - DO NOT EDIT !!
// generated by 'emugen'
#ifndef __GUARD_foo_opcodes_h_
#define __GUARD_foo_opcodes_h_

#define OP_fooAlphaFunc 					200
#define OP_fooIsBuffer 					201
#define OP_fooUnsupported 					202
#define OP_fooDoEncoderFlush 					203
#define OP_fooTakeConstVoidPtrConstPtr 					204
#define OP_last 					205


#endif
